#!/bin/sh
# recentfiles.sh 1.1 - (C) 15 Jun 2016 GPL3, Roger D. Grider (radky)
#
# This script parses recently-used.xbel & creates a list of files recently
# opened by GTK open/save dialogs. Based on an openbox pipe menu script by
# John Crawley (recently_opened_menu.sh copyright 2010) with modifications
# including display of full application path, automatic exclusion of files
# that no longer exist, selectable number of files, and optional open cmd.
# Original source is available, http://crunchbang.org/pastebin/?paste=1506

export TEXTDOMAIN=recentfiles
export OUTPUT_CHARSET=UTF-8

APPDIR="$HOME/.config/openbox"

MIME_CMD=$(cat $APPDIR/mime-command 2>/dev/null)
[ ! "$MIME_CMD" ] && MIME_CMD="false"

# maximum number of files parsed in recently-used.xbel
PARSEDFILES="50"

# maximum number of files shown in menu (exclude non-existent files in $PARSEDFILES)
MAX_FILES=$(cat $APPDIR/max-files 2>/dev/null)
[ ! "$MAX_FILES" ] && MAX_FILES="15"

# optional open command
[ "`which xdg-open`" ] && OPEN="xdg-open" || OPEN="exo-open"
export OPEN

# path of recently-used.xbel
if [ $XDG_DATA_HOME ] && [ -r "${XDG_DATA_HOME}/recently-used.xbel" ]
then
    file_path="${XDG_DATA_HOME}/recently-used.xbel"
elif [ -r "${HOME}/.local/share/recently-used.xbel" ]
then
    file_path="${HOME}/.local/share/recently-used.xbel"
elif [ -r "${HOME}/.recently-used.xbel" ]
then
    file_path="${HOME}/.recently-used.xbel"
else
    echo "$0: cannot find a readable recently-used.xbel file" >&2
    echo '<openbox_pipe_menu>
<separator label="'"$(gettext 'No recently-used.xbel file found')"'" />
</openbox_pipe_menu>'
    exit 1
fi

# if argument is --clear, remove old file and create new recently-used.xbel
[ "$1" = '--clear' ] && {
Xdialog --title "$(gettext 'Recent Files')" --cancel-label "$(gettext 'No')" --ok-label "$(gettext 'Yes')"  --yesno "\n $(gettext 'Start new list of recent files?') \n\n $(gettext 'This will delete the current recently-used.xbel.') \n" 0 0
if [[ $? == 0 ]]; then
cat <<':EOF' > "${file_path}"
<?xml version="1.0" encoding="UTF-8"?>
<xbel version="1.0"
      xmlns:bookmark="http://www.freedesktop.org/standards/desktop-bookmarks"
      xmlns:mime="http://www.freedesktop.org/standards/shared-mime-info"
>
</xbel>
:EOF
fi
exit
}

# if argument is --opencommand, set default open command
[ "$1" = '--opencommand' ] && {
if [ "$MIME_CMD" = "true" ]; then
  REPLY=$(Xdialog --stdout --title "$(gettext 'Recent Files')" --no-tags --no-cancel --radiolist "$(gettext 'Open Command:') $OPEN" 10 60 2  "1" "$OPEN $(gettext 'command based on the file MIME type')" "on"  "2" "$(gettext 'Original command defined in recently-used.xbel')" "off" )
  if [ "$REPLY" = "1" ]; then
    continue
  elif [ "$REPLY" = "2" ]; then
    echo "false" > $APPDIR/mime-command
  else
    exit 1
  fi
else 
  REPLY=$(Xdialog --stdout --title "$(gettext 'Recent Files')" --no-tags --no-cancel --radiolist "$(gettext 'Open Command:') original" 10 70 2  "1" "$(gettext 'Original command defined in recently-used.xbel')" "on"  "2" "$OPEN $(gettext 'command based on the file MIME type')" "off" )
  if [ "$REPLY" = "1" ]; then
    continue
  elif [ "$REPLY" = "2" ]; then
    echo "true" > $APPDIR/mime-command
  else
    exit 1
  fi
fi
exit
}

# if argument is --filenumber, set maximum number of recent files
[ "$1" = '--filenumber' ] && {
REPLY=$(Xdialog --stdout --title "$(gettext 'Recent Files')" --no-cancel --rangebox "$(gettext 'Maximum Number of Files')" 10 40 10 50 $MAX_FILES )
if [ "$REPLY" = "" ]; then
  exit 1
else
  echo "$REPLY" > $APPDIR/max-files
fi
exit
}

PARSEDFILES=$((PARSEDFILES+2))

files=$( tac "${file_path}" | sed \
    -e "s/;gtkdialog4/;$OPEN/g" \
    -e "s/;gtkdialog3/;$OPEN/g" \
    -e "s/;gtkdialog/;$OPEN/g" | \
    awk -v MAX="$PARSEDFILES" 'BEGIN {
    RS="</bookmark>";
    FS="<info>";
}
(NR == MAX) {exit}
!/<bookmark/ {next}
!/href=[\"'\'']file:\/\// {next}
# $1 is the command, $2 the file path
{
    sub(/^.*exec=\"\&apos\;/,"",$1)
    sub(/\&apos\;.*$/,"",$1)
    sub(/ *%./,"",$1)
    sub(/^.*file:\/\//,"",$2)
    sub(/[\"'\''].*$/,"",$2)
    gsub(/%22/,"\\&quot;",$2)
    gsub(/%3C/,"\\&lt;",$2)
    gsub(/%3E/,"\\&gt;",$2)
    name=$2
    sub(/^.*\//,"",name)
    gsub(/_/,"__",name)
    gsub(/\&apos;/,"\\&apos;\\&quot;\\&apos;\\&quot;\\&apos;",$2)
    print ($1 " " $2 "")
}' )

# use perl to decode urlencoded characters
files=$(perl -MURI::Escape -e 'print uri_unescape($ARGV[0]);' "$files")

# parse output and exclude recently-used files that no longer exist
printf '%s' "$files" | awk '!a[$0]++' > /tmp/tmp
echo -n > /tmp/tmp1
 while read -r line
  do
    [ "$MIME_CMD" = "true" ] && COMMAND="$OPEN" || COMMAND=$(echo "$line" | cut -d' ' -f1)
    FILE_PATH=$(echo "$line" | cut -d' ' -f2-)
    if [ -s "$FILE_PATH" ]; then # file exists, add corresponding data triad to openbox pipe menu
      echo -e "    <item label="'"'$FILE_PATH'"'">" >> /tmp/tmp1
      echo -e "    <action name="'"'Execute'"'"><command>"'"'$COMMAND'"'" '"$FILE_PATH"'</command></action>" >> /tmp/tmp1
      echo -e "    </item>" >> /tmp/tmp1
    fi
  done < /tmp/tmp

MAX_LINES=$(echo "$(($MAX_FILES * 3))")
files=$(head -n $MAX_LINES /tmp/tmp1 2>/dev/null) # extract corresponding data triad for each parsed file

[ "$MIME_CMD" = "true" ] && OPENCOMMAND="$OPEN" || OPENCOMMAND="original"

output='<openbox_pipe_menu>
'"$files"'
<separator/>
    <item label=" ▸ '"$(gettext 'Set maximum files')"' ('"$MAX_FILES"')">
        <action name="Execute">
            <command>
            &apos;'"$0"'&apos; --filenumber
            </command>
        </action>
    </item>
    <item label=" ▸ '"$(gettext 'Set open command')"' ('"$OPENCOMMAND"')">
        <action name="Execute">
            <command>
            &apos;'"$0"'&apos; --opencommand
            </command>
        </action>
    </item>
<separator/>
    <item label="✖ '"$(gettext 'Clear recent files')"'">
        <action name="Execute">
            <command>
            &apos;'"$0"'&apos; --clear
            </command>
        </action>
    </item>
</openbox_pipe_menu>
'
printf '%s' "$output"
